diff -u -r1.1 chan_skinny.c
--- channels/chan_skinny.c	11 Sep 2003 19:44:13 -0000	1.1
+++ channels/chan_skinny.c	12 Sep 2003 20:09:41 -0000
@@ -1716,7 +1716,12 @@
 	time_t timer;
 	struct tm *cmtime;
 	pthread_t t;
-		
+
+	if ( (!s->device) && (req->e != REGISTER_MESSAGE && req->e != REGISTER_MESSAGE)) {
+		ast_log(LOG_WARNING, "Client sent message #%d without first registering.\n", req->e);
+		free(req);
+		return 0;
+	}
 		
 	switch(req->e)	{
 	case ALARM_MESSAGE:
@@ -2135,23 +2140,25 @@
  
 	res = ast_select(s->fd + 1, &fds, NULL, NULL, NULL);  
  
-	if (res < 0) {  
-		ast_log(LOG_WARNING, "Select returned error: %s\n", strerror(errno));  
- 	} else if (res > 0) {  
-		ast_mutex_lock(&s->lock);  
+	if (res < 0) {
+		ast_log(LOG_WARNING, "Select returned error: %s\n", strerror(errno));
+ 	} else if (res > 0) {
+		ast_mutex_lock(&s->lock);
 		memset(s->inbuf,0,sizeof(s->inbuf));
-		res = read(s->fd, s->inbuf, 4);  
-		dlen = *(int *)s->inbuf; 
-		if (res < 1) {  
-			return -1;  
-		}  
-		res = read(s->fd, s->inbuf+4, dlen+4);  
-		ast_mutex_unlock(&s->lock);  
-		if (res < 1) {  
-			return -1;  
-		}  
+		res = read(s->fd, s->inbuf, 4);
+		if (res != 4) {
+			ast_log(LOG_WARNING, "Skinny Client sent less data than expected.\n");
+			return -1;
+		}
+		dlen = *(int *)s->inbuf;
+		res = read(s->fd, s->inbuf+4, dlen+4);
+		ast_mutex_unlock(&s->lock);
+		if (res != (dlen+4)) {
+			ast_log(LOG_WARNING, "Skinny Client sent less data than expected.\n");
+			return -1;
+		}
  
-	}  
+	}
 	return res;  
 }   
 
@@ -2165,10 +2172,13 @@
 		ast_log(LOG_ERROR, "Unable to allocate skinny_request, this is bad\n");
 		return NULL;
 	}
+
 	memset(req, 0, sizeof(skinny_req));
 	memcpy(req, s->inbuf, *(int*)(s->inbuf)+8); // +8
+
 	if (req->e < 0) {
 		ast_log(LOG_ERROR, "Event Message is NULL from socket %d, This is bad\n", s->fd);
+		free(req);
 		return NULL;
 	}
 	return req;
